/*****************************************************************************
 * $Id: eng_defs.hh,v 1.12 2006/11/08 00:59:13 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Definitions, constants, settings.
 * @remarks @verbatim
 * ----------------------------------------------------------------------------
 * This header file should only contain self-contained constants (no dependencies).
 * C++ const is preferred to use type-checking but some must be #defined
 * as some source files test them using #if.
 * NOTE: Be sure to WRITE A PERIOD IN FLOAT CONSTANTS defined by #define
 * or integer instructions could be compiled (with loss of accuracy).
 * ----------------------------------------------------------------------------
 * @endverbatim
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef ENG_DEFS_HH
#define ENG_DEFS_HH 1

//------------------------------------------------------------------------------
// Settings:
#define		DEFAULT_TIMER_MILSEC		20
#define		MAIN_WINDOW_WIDTH			600
#define		MAIN_WINDOW_HEIGHT			600
#define     MAX_POLYGON_VERTEXS_BITS    3					///< amount of bits in MAX_POLYGON_VERTEXS
#define		MAX_POLYGON_VERTEXS			6					///< should be very small
#define		MAX_VERTEXS					32767
#define		VIX_NONE					MAX_VERTEXS
#define		MAX_POLYGONS				32767
#define		LOD_TEXTURE_LEAST			(eLodUser_MID)      ///< textures skipped if object has less LOD
#define		MAX_OBJECT_RADIUS			WORLD(MEGABYTE)
#define     MIN_OBJECT_RADIUS           0                   ///< 0 for abstract objects without geometry
#define     MIN_OBJECT_SORT_DISTANCE    5.0                 ///< Objects less than this are culled
#define     MAX_OBJECT_DISTANCE         8000000.0           ///< to catch garbage values
//------------------------------------------------------------------------------

// Specify an unused vertex (in the X coord).
#ifndef NOVERTEX
#define NOVERTEX (12345678.0)	// DEPRECATED
#endif

#define AXIS_PITCH AXIS_X
#define AXIS_YAW   AXIS_Y
#define AXIS_ROLL  AXIS_Z

#endif // ENG_DEFS_HH
